<?php
	class Sudoku {
		private $board = array(array());
		private $isValid = false;
		private $isSolved = false;
		private $numChecks = 0;
		private $numCols	= 9;
		private $numRows	= 9;
		private $lastRow	= 9;
		private $boxFields	= 3;
		public function __construct() {
			$this->reset();
		}
		public function solve() {
			$this->setPossibleValues();
			$startRow = 0;
			while (!$this->isValid) {
				$this->numChecks = 0;
				$this->isValid = $this->solveBacktracking($startRow, 0);
				$this->lastRow--;
				$startRow = $this->lastRow;
				if ($this->lastRow == 0) return false;
			}
			$this->isValid = $this->validateBoard();
			$this->isSolved = $this->isValid;
			return true;
		}
		public function reset() {
			for ($row = 0; $row < $this->numRows; $row++) {
				for ($col = 0; $col < $this->numCols; $col++) {
					$this->board[$row][$col] = new SudokuField();
				}
			}
		}
		private function setPossibleValues() {
			for ($row = 0; $row < $this->numRows; $row++) {
				for ($col = 0; $col < $this->numCols; $col++) {
					$field = $this->board[$row][$col];
					if (is_null($field->getValue())) {
						$possibleValues = array();
						for ($testValue = 1; $testValue <= 9; $testValue++) {
							if ($this->check($row, $col, $testValue)) $possibleValues[] = $testValue;
						}
						$field->setPossibleValues($possibleValues);
					}
				}
			}
		}
		private function solveBacktracking($row, $col) {
				if ($this->numChecks > 4000) return false;
				if ($col == $this->numCols) {
					if (($row+1) == $this->lastRow) return true;
					$col = 0;
					$row = $row+1;
					if ($row == $this->numRows) $row = 0;
				}
				$field = $this->board[$row][$col];
				if (!is_null($field->getValue())) {
					return $this->solveBacktracking($row, $col+1);
				}
				foreach ($field->getPossibleValues() as $testValue) {
					if ($this->check($row, $col, $testValue)) {
						$field->setValue($testValue);
						if ($this->solveBacktracking($row, $col+1)) return true;
					}
				}
			$field->setValue(null);
			return false;
		}
		public function check($row, $col, $value, $section = 'all') {
			$this->numChecks++;
			switch ($section) {
				case 'all':
					if (!$this->checkRow($row, $value)) return false;
					if (!$this->checkCol($col, $value)) return false;
					if (!$this->checkBox($row, $col, $value)) return false;
					break;
				case 'row':
					if (!$this->checkRow($row, $value)) return false;
					break;
				case 'col':
					if (!$this->checkCol($col, $value)) return false;
					break;
				case 'box':
					if (!$this->checkBox($row, $col, $value)) return false;
					break;
			}
			return true;
		}
		private function checkRow($row, $value) {
			for ($testCol = 0; $testCol < $this->numCols; $testCol++) {
				$testField = $this->board[$row][$testCol];
				if (!is_null($testField->getValue())) {
					if ($testField->getValue() == $value) return false;
				}
			}
			return true;
		}
		private function checkCol($col, $value) {
			for ($testRow = 0; $testRow < $this->numRows; $testRow++) {
				$testField = $this->board[$testRow][$col];
				if (!is_null($testField->getValue())) {
					if ($testField->getValue() == $value) return false;
				}
			}
			return true;
		}
		private function checkBox($row, $col, $value) {
			$rowStart = ($row - ($row%$this->boxFields));
			$colStart = ($col - ($col%$this->boxFields));
			for ($testRow = $rowStart; $testRow < ($rowStart+$this->boxFields); $testRow++) {
				for ($testCol = $colStart; $testCol < ($colStart+$this->boxFields); $testCol++ ) {
					$testField = $this->board[$testRow][$testCol];
					if (!is_null($testField->getValue())) {
						if (($testRow != $row && $testCol != $col) && $testField->getValue() == $value) {
							return false;
						}
					}
				}
			}
			return true;
		}
		public function validateBoard() {
			for ($row = 0; $row < $this->numRows; $row++) {
				for ($col = 0; $col < $this->numCols; $col++) {
					$field = $this->board[$row][$col];
					if (!is_null($field->getValue())) {
						if (!$this->validate($row, $col)) return false;
					}
				}
			}
			$this->isValid = true;
			return true;
		}
		public function validate($row, $col, $section = 'all') {
			switch ($section) {
				case 'all':
					if (!$this->validateRow($row, $col)) return false;
					if (!$this->validateCol($row, $col)) return false;
					if (!$this->validateBox($row, $col)) return false;
					break;
				case 'row':
					if (!$this->validateRow($row, $col)) return false;
					break;
				case 'col':
					if (!$this->validateCol($row, $col)) return false;
					break;
				case 'box':
					if (!$this->validateBox($row, $col)) return false;
					break;
			}
			return true;
		}
		private function validateRow($row, $col) {
			$field = $this->board[$row][$col];
			
			for($testCol = 0; $testCol < $this->numCols; $testCol++) {
				$testField = $this->board[$row][$testCol];
				if (!is_null($testField->getValue())) {
					if(($testCol != $col) && $testField->getValue() == $field->getValue()) {
						$field->setIsValid(false);
						$testField->setIsValid(false);
						return false;
					}
				}
			}
			return true;
		}
		private function validateCol($row, $col) {
			$field = $this->board[$row][$col];
			
			for($testRow = 0; $testRow < $this->numRows; $testRow++) {
				$testField = $this->board[$testRow][$col];
				if (!is_null($testField->getValue())) {
					if(($testRow != $row) && $testField->getValue() == $field->getValue()) {
						$field->setIsValid(false);
						$testField->setIsValid(false);
						return false;
					}
				}
			}
			return true;
		}
		private function validateBox($row, $col) {
			$field = $this->board[$row][$col];
			$rowStart = ($row - ($row%$this->boxFields));
			$colStart = ($col - ($col%$this->boxFields));
			for ($testRow = $rowStart; $testRow < ($rowStart+$this->boxFields); $testRow++) {
				for ($testCol = $colStart; $testCol < ($colStart+$this->boxFields); $testCol++ ) {
					$testField = $this->board[$testRow][$testCol];
					if (!is_null($testField->getValue())) {
						if (($testRow != $row && $testCol != $col) && $testField->getValue() == $field->getValue()) {
							$field->setIsValid(false);
							$testField->setIsValid(false);
							return false;
						}
					}
				}
			}
			return true;
		}
		public function generate($numRandomFields) {
			$row = rand(0,8);
			$col = rand(0,8);
			$rowStart = ($row - ($row%$this->boxFields));
			$colStart = ($col - ($col%$this->boxFields));
			$availableValues = array(1,2,3,4,5,6,7,8,9);
			for ($row = $rowStart; $row < ($rowStart+$this->boxFields); $row++) {
				for ($col = $colStart; $col < ($colStart+$this->boxFields); $col++ ) {
					$field = $this->board[$row][$col];
					$index = rand(0, sizeof($availableValues)-1);
					$field->setValue($availableValues[$index]);
					$field->setIsValid(true);
					array_splice($availableValues, $index, 1);
				}
			}
			$this->solve();
			for ($index = 0; $index < $numRandomFields; $index++) {
				$searchField = true;
				while ($searchField) {
					$randField	= $this->board[rand(0,8)][rand(0,8)];
					if (!$randField->getIsGiven()) {
						$randField->setIsGiven(true);
						$searchField = false;
					}
				}
			}
		}
		public function setField($row, $col, $value) {
			if ($row >= 0 && $row < ($this->numRows) && $col >= 0 && $col < ($this->numCols) && $value > 0 && $value <= 9) {
				$field = $this->board[$row][$col];
				
				$field->setValue($value);
				$field->setIsGiven(true);
			}
		}
		public function setBoard($value) {
			$this->board = $value;
		}
		public function getBoard() {
			return $this->board;
		}
		public function setIsValid($value) {
			$this->isValid = $value;
		}
		public function getIsValid() {
			return $this->isValid;
		}
		public function setIsSolved($value) {
			$this->isSolved = $value;
		}
		public function getIsSolved() {
			return $this->isSolved;
		}
		public function setNumChecks($value) {
			$this->numChecks = $value;
		}
		public function getNumChecks() {
			return $this->numChecks;
		}
	}
?>