<?php
	require_once 'classes/Sudoku.php';
	require_once 'classes/SudokuField.php';
	require_once 'classes/Helper.php';
	$defaultLanguage	= 'de';
	$defaultRndFields	= 30;
	$numRows			= 9;
	$numCols			= 9;
	$cellClass			= 'cellEven';
	$helper				= new Helper();
	$sudoku				= new Sudoku();
	$action		= preg_replace('/[^a-z]/', '', $_REQUEST['action']);
	switch ($_REQUEST['lang']) {
		case 'de': $language = 'de'; break;
		case 'en': $language = 'en'; break;
		case 'fr': $language = 'fr'; break;
		default: $language = $defaultLanguage; break;
	}
	$helper->loadLanguage($language);
	if ($action == 'calculate' || $action == 'check') {
		for ($row = 0; $row < $numRows; $row++) {
			for ($col = 0; $col < $numCols; $col++) {
				$value = preg_replace('/[^1-9]/', '', $_POST['field_'.$row.'_'.$col]);
				$sudoku->setField($row, $col, $value);
			}
		}
		if ($action == 'calculate') {
			$sudoku->solve();
		} else {
			$sudoku->validateBoard();
		}
	} else if ($action == 'generate') {
		$sudoku->generate(preg_replace('/[^0-9]/', '', $_POST['numRandomFields']));
	}
	$sudokuBoard = $sudoku->getBoard(); 
?>
<!DOCTYPE html><!-- HTML5 -->
<html prefix="og: http://ogp.me/ns#" lang="de-DE" dir="ltr">
	<head>
		<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
		<title><?php echo LANG_PAGE_TITLE ?></title>
		<link rel="stylesheet" type="text/css" media="screen" href="style.css" />
		<script type="text/javascript" src="functions.js"></script>
	</head>
	<body>
		<div class="mainContainer">
			<form action="index.php" method="post">
				<div class="header">
				</div>
				<div class="leftsigns"></div>
				<div style="float: left; width: 360px;">
					<table cellspacing="0">
						<?php for ($row = 0; $row < $numRows; $row++): ?>
							<?php $cellClass = ($row > 2 && $row < 6) ? 'cellOdd' : 'cellEven'; ?>
							<tr>
								<?php for ($col = 0; $col < $numCols; $col++): ?>
									<?php $field = $sudokuBoard[$row][$col] ?>
									<?php $cellClass = ($col % 3 == 0) ? $helper->switchCellClass($cellClass) : $cellClass; ?>
									<td class="<?php echo $cellClass ?>">
										<input type="text" maxlength="1" class="inputText_<?php echo $cellClass ?>" 
										name="field_<?php echo $row.'_'.$col ?>" id="field_<?php echo $row.'_'.$col ?>" 
										value="<?php echo ($field->getIsGiven()) ? $field->getValue() : '' ?>" 
										onkeyup="checkField('<?php echo $row ?>', '<?php echo $col ?>');" />
									</td>
								<?php endfor ?>
							</tr>
						<?php endfor ?>
					</table>
				</div>
				<div class="rightsigns"></div>
				<div style="clear: both;"></div>
				<p style="margin: 4px auto;">
					<input type="button" value="<?php echo BUTTON_GENERATE ?>" onclick="confirmGenerate();" />
					<input type="button" value="<?php echo BUTTON_CHECK ?>" onclick="checkSudoku();" />
					<input type="submit" value="<?php echo BUTTON_SOLVE ?>" />
					<input type="button" value="<?php echo BUTTON_RESET ?>" onclick="confirmReset();" />
					<input type="hidden" name="action" id="action" value="calculate" />
					<input type="hidden" name="numRandomFields" id="numRandomFields" value="0" />
					<input type="hidden" name="lang" id="lang" value="<?php echo $language ?>" />
				</p>
			</form>
			<?php if ($action == 'calculate' || $action == 'check'): ?>
				<div class="infoBox">
					<img src="images/information.png" style="float: left; margin-right: 3px;" alt="Info" />
					<?php if ($action == 'calculate' && $sudoku->getIsSolved()): ?>
						<?php echo MSG_SUDOKU_IS_SOLVABLE ?>
					<?php elseif ($action == 'calculate' && !$sudoku->getIsSolved()): ?>
						<?php echo MSG_SUDOKU_IS_NOT_SOLVABLE ?>
					<?php elseif ($action == 'check' && $sudoku->getIsValid()): ?>
						<?php echo MSG_SUDOKU_IS_VALID ?>
					<?php elseif ($action == 'check' && !$sudoku->getIsValid()): ?>
						<?php echo MSG_SUDOKU_IS_NOT_VALID ?>
					<?php endif ?>
				</div>
				<?php if (($action == 'calculate' && $sudoku->getIsSolved()) || ($action == 'check')): ?>
					<table cellspacing="0">
						<?php for ($row = 0; $row < $numRows; $row++): ?>
							<?php $cellClass = ($row > 2 && $row < 6) ? 'cellOdd' : 'cellEven'; ?>
							<tr>
								<?php for ($col = 0; $col < $numCols; $col++): ?>
										<?php $field = $sudokuBoard[$row][$col] ?>
									<?php $cellClass = ($col % 3 == 0) ? $helper->switchCellClass($cellClass) : $cellClass; ?>
									<td class="<?php echo $cellClass ?>" <?php if (!$field->getIsValid()) echo 'style="color: red;"' ?>>
										<?php echo (!is_null($field->getValue())) ? $field->getValue() : '&nbsp;' ?>
									</td>
								<?php endfor ?>
							</tr>
						<?php endfor ?>
					</table>
				<?php endif ?>
			<?php else: ?>
				<div class="infoBox">
					<img src="images/information.png" style="float: left; margin-right: 3px;" alt="Info" />
					<p style="font-weight: bold; margin-bottom: 6px;"><?php echo MSG_WELCOME_PART1 ?></p>
					<p class="welcomeMsg"><?php echo MSG_WELCOME_PART2 ?></p>
					<p class="welcomeMsg"><?php echo MSG_WELCOME_PART3 ?></p>
				</div>
			<?php endif ?>
		</div>
		<script>
			function confirmLangChange() {
				return confirm("<?php echo MSG_CONFIRM_LANGUAGE_CHANGE ?>");
			}
			function confirmGenerate() {
				var myRandomFields = prompt("<?php echo MSG_PROMPT_NUM_RANDOM_FIELDS ?>", "<?php echo $defaultRndFields ?>");
				if (myRandomFields.length > 0 && myRandomFields.length < 3) {
					document.getElementById('numRandomFields').value = myRandomFields;
					document.getElementById('action').value = 'generate';
					document.forms[0].submit();
				}
			}
			function confirmReset() {
				if (confirm("<?php echo MSG_CONFIRM_RESET ?>")) {
					location.href = 'index.php?lang=<?php echo $language ?>';
				}
			}
			function checkSudoku() {
				document.getElementById('action').value = 'check';
				document.forms[0].submit();
			}
		</script>
	</body>
</html>